use serde_yaml;
use serde::{Deserialize, Serialize};
use clap::Parser;
use tokio;
use log::*;
use chrono::prelude::*;
use frankenstein;
use frankenstein::client_reqwest::Bot;
use frankenstein::AsyncTelegramApi;
use frankenstein::SendMessageParams;

#[derive(Debug, PartialEq, Serialize, Deserialize)]
struct Config {
    token: String,
    chatid: i64,
}

#[derive(Parser, Debug)]
#[clap(author, version, about, long_about = None)]
struct Opts {
    /// Config fle
    #[clap(long)]
    config: Option<String>,
    /// Telegram token
    #[clap(long)]
    token: Option<String>,
    /// Telegram chat ID
    #[clap(long, allow_hyphen_values(true))]
    chatid: Option<i64>,
    /// Message to send
    #[clap(long)]
    msg: String,
    /// Verbose level, accept multiple occurrences
    #[clap(short, long, action = clap::ArgAction::Count)]
    verbose: u8,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let opts: Opts = Opts::parse();

    stderrlog::new()
        .module(module_path!())
        .verbosity(usize::from(opts.verbose))
        .timestamp(stderrlog::Timestamp::Second)
        .quiet(false)
        .init()
        .unwrap();
    
    if let (Some(t),Some(c)) = (opts.token.clone(), opts.chatid.clone()) {
        let config = Config {
            token: t,
            chatid: c,
        };
        let _ = telegram_notify(&opts, &config).await?;

    } else if let Some(_config) = opts.config.as_ref() {
        println!("{} - LOG - Config file name: {}", Local::now().trunc_subsecs(0).to_rfc3339(), &opts.config.as_ref().unwrap());
        let f = std::fs::File::open(&opts.config.as_ref().unwrap())?;
        let config: Config = serde_yaml::from_reader(f)?;
        let _ = telegram_notify(&opts, &config).await?;

    } else {
        println!("{} - LOG - Config file name: {}", Local::now().trunc_subsecs(0).to_rfc3339(), ".telegram-notifier.yaml");
        let f = std::fs::File::open(".telegram-notifier.yaml").expect("Can't openf config file");
        let config: Config = serde_yaml::from_reader(f)?;
        let _ = telegram_notify(&opts, &config).await?;
    }

    Ok(())
}

async fn telegram_notify(opts: &Opts, config: &Config) -> Result<(), Box<dyn std::error::Error>> {
    debug!("Using token: {}", config.token);
    println!("{} - LOG - Using chat id: {}", Local::now().trunc_subsecs(0).to_rfc3339(), config.chatid);
    println!("{} - LOG - {}", Local::now().trunc_subsecs(0).to_rfc3339(), opts.msg);

    let api = Bot::new(&config.token);
    
    let str = Local::now().format("%Y-%m-%d %H:%M:%S: ").to_string() + &opts.msg.to_string();
    let send_message_params: SendMessageParams = SendMessageParams::builder()
        .chat_id(config.chatid)
        .text(str)
        .build();

    if let Err(err) = api.send_message(&send_message_params).await {
        println!("Failed to send message: {err:?}");
    }

    Ok(())
}
